@@ -1,6 +1,8 @@
 use std::io;
 
-use super::{BLOB, COMMIT, OFS_DELTA, REF_DELTA, SHA1_SIZE, TAG, TREE};
+use git_features::decode::{leb64, leb64_from_read};
+
+use super::{BLOB, COMMIT, OFS_DELTA, REF_DELTA, TAG, TREE};
 use crate::data;
 
 /// Decoding
@@ -10,13 +12,13 @@ impl data::Entry {
     /// # Panics
     ///
     /// If we cannot understand the header, garbage data is likely to trigger this.
-    pub fn from_bytes(d: &[u8], pack_offset: u64) -> data::Entry {
+    pub fn from_bytes(d: &[u8], pack_offset: data::Offset, hash_len: usize) -> data::Entry {
         let (type_id, size, mut consumed) = parse_header_info(d);
 
         use crate::data::entry::Header::*;
         let object = match type_id {
             OFS_DELTA => {
-                let (distance, leb_bytes) = leb64decode(&d[consumed..]);
+                let (distance, leb_bytes) = leb64(&d[consumed..]);
                 let delta = OfsDelta {
                     base_distance: distance,
                 };
@@ -25,9 +27,9 @@ impl data::Entry {
             }
             REF_DELTA => {
                 let delta = RefDelta {
-                    base_id: git_hash::ObjectId::from_20_bytes(&d[consumed..consumed + SHA1_SIZE]),
+                    base_id: git_hash::ObjectId::from(&d[consumed..][..hash_len]),
                 };
-                consumed += SHA1_SIZE;
+                consumed += hash_len;
                 delta
             }
             BLOB => Blob,
@@ -44,13 +46,17 @@ impl data::Entry {
     }
 
     /// Instantiate an `Entry` from the reader `r`, providing the `pack_offset` to allow tracking the start of the entry data section.
-    pub fn from_read(mut r: impl io::Read, pack_offset: u64) -> Result<data::Entry, io::Error> {
+    pub fn from_read(
+        mut r: impl io::Read,
+        pack_offset: data::Offset,
+        hash_len: usize,
+    ) -> Result<data::Entry, io::Error> {
         let (type_id, size, mut consumed) = streaming_parse_header_info(&mut r)?;
 
         use crate::data::entry::Header::*;
         let object = match type_id {
             OFS_DELTA => {
-                let (distance, leb_bytes) = streaming_leb64decode(&mut r)?;
+                let (distance, leb_bytes) = leb64_from_read(&mut r)?;
                 let delta = OfsDelta {
                     base_distance: distance,
                 };
@@ -58,12 +64,13 @@ impl data::Entry {
                 delta
             }
             REF_DELTA => {
-                let mut buf = [0u8; SHA1_SIZE];
-                r.read_exact(&mut buf)?;
+                let mut buf = git_hash::Kind::buf();
+                let hash = &mut buf[..hash_len];
+                r.read_exact(hash)?;
                 let delta = RefDelta {
-                    base_id: git_hash::ObjectId::new_sha1(buf),
+                    base_id: git_hash::ObjectId::from(&hash[..]),
                 };
-                consumed += SHA1_SIZE;
+                consumed += hash_len;
                 delta
             }
             BLOB => Blob,
@@ -80,37 +87,6 @@ impl data::Entry {
     }
 }
 
-#[inline]
-fn streaming_leb64decode(mut r: impl io::Read) -> Result<(u64, usize), io::Error> {
-    let mut b = [0u8; 1];
-    let mut i = 0;
-    r.read_exact(&mut b)?;
-    i += 1;
-    let mut value = b[0] as u64 & 0x7f;
-    while b[0] & 0x80 != 0 {
-        r.read_exact(&mut b)?;
-        i += 1;
-        value += 1;
-        value = (value << 7) + (b[0] as u64 & 0x7f)
-    }
-    Ok((value, i))
-}
-
-#[inline]
-fn leb64decode(d: &[u8]) -> (u64, usize) {
-    let mut i = 0;
-    let mut c = d[i];
-    i += 1;
-    let mut value = c as u64 & 0x7f;
-    while c & 0x80 != 0 {
-        c = d[i];
-        i += 1;
-        value += 1;
-        value = (value << 7) + (c as u64 & 0x7f)
-    }
-    (value, i)
-}
-
 #[inline]
 fn streaming_parse_header_info(mut read: impl io::Read) -> Result<(u8, u64, usize), io::Error> {
     let mut byte = [0u8; 1];
