@@ -49,32 +49,28 @@ where
 {
     /// Create a new instance reading [entries][output::Entry] from an `input` iterator and write pack data bytes to
     /// `output` writer, resembling a pack of `version` with exactly `num_entries` amount of objects contained in it.
-    /// `hash_kind` is the kind of hash to use for the pack checksum and maybe other places, depending on the version.
+    /// `object_hash` is the kind of hash to use for the pack checksum and maybe other places, depending on the version.
     ///
-    /// The input chunks are expected to be sorted already. You can use the [InOrderIter][super::InOrderIter] to assure
+    /// The input chunks are expected to be sorted already. You can use the [InOrderIter][git_features::parallel::InOrderIter] to assure
     /// this happens on the fly holding entire chunks in memory as long as needed for them to be dispensed in order.
     ///
     /// # Panics
     ///
-    /// Not all combinations of `hash_kind` and `version` are supported currently triggering assertion errors.
+    /// Not all combinations of `object_hash` and `version` are supported currently triggering assertion errors.
     pub fn new(
         input: I,
         output: W,
         num_entries: u32,
         version: crate::data::Version,
-        hash_kind: git_hash::Kind,
+        object_hash: git_hash::Kind,
     ) -> Self {
         assert!(
             matches!(version, crate::data::Version::V2),
             "currently only pack version 2 can be written",
         );
-        assert!(
-            matches!(hash_kind, git_hash::Kind::Sha1),
-            "currently only Sha1 is supported, right now we don't know how other hashes are encoded",
-        );
         FromEntriesIter {
             input,
-            output: hash::Write::new(output, hash_kind),
+            output: hash::Write::new(output, object_hash),
             trailer: None,
             entry_version: version,
             pack_offsets_and_validity: Vec::with_capacity(num_entries as usize),
