@@ -27,7 +27,6 @@ pub type Result<E1, E2> = std::result::Result<(Vec<output::Count>, Outcome), Err
 /// A [`Count`][output::Count] object maintains enough state to greatly accelerate future access of packed objects.
 ///
 /// * `db` - the object store to use for accessing objects.
-/// * `make_cache` - a function to create thread-local pack caches
 /// * `objects_ids`
 ///   * A list of objects ids to add to the pack. Duplication checks are performed so no object is ever added to a pack twice.
 ///   * Objects may be expanded based on the provided [`options`][Options]
@@ -37,9 +36,8 @@ pub type Result<E1, E2> = std::result::Result<(Vec<output::Count>, Outcome), Err
 ///  * A flag that is set to true if the operation should stop
 /// * `options`
 ///   * more configuration
-pub fn objects<Find, Iter, IterErr, Oid, PackCache, ObjectCache>(
+pub fn objects<Find, Iter, IterErr, Oid>(
     db: Find,
-    make_caches: impl Fn() -> (PackCache, ObjectCache) + Send + Clone,
     objects_ids: Iter,
     progress: impl Progress,
     should_interrupt: &AtomicBool,
@@ -55,8 +53,6 @@ where
     Iter: Iterator<Item = std::result::Result<Oid, IterErr>> + Send,
     Oid: Into<ObjectId> + Send,
     IterErr: std::error::Error + Send,
-    PackCache: crate::cache::DecodeEntry,
-    ObjectCache: crate::cache::Object,
 {
     let lower_bound = objects_ids.size_hint().0;
     let (chunk_size, thread_limit, _) = parallel::optimize_chunk_size_and_thread_limit(
@@ -79,9 +75,8 @@ where
             let progress = Arc::clone(&progress);
             move |n| {
                 (
-                    Vec::new(),    // object data buffer
-                    Vec::new(),    // object data buffer 2 to hold two objects at a time
-                    make_caches(), // cache to speed up pack and tree traveral operations
+                    Vec::new(), // object data buffer
+                    Vec::new(), // object data buffer 2 to hold two objects at a time
                     {
                         let mut p = progress.lock().add_child(format!("thread {}", n));
                         p.init(None, git_features::progress::count("objects"));
@@ -91,19 +86,18 @@ where
             }
         },
         {
-            move |oids: Vec<std::result::Result<Oid, IterErr>>, (buf1, buf2, (pack_cache, object_cache), progress)| {
+            let seen_objs = &seen_objs;
+            move |oids: Vec<std::result::Result<Oid, IterErr>>, (buf1, buf2, progress)| {
                 expand::this(
                     &db,
                     input_object_expansion,
-                    &seen_objs,
+                    seen_objs,
                     oids,
                     buf1,
                     buf2,
-                    pack_cache,
-                    object_cache,
                     progress,
                     should_interrupt,
-                    true,
+                    true, /*allow pack lookups*/
                 )
             }
         },
@@ -114,7 +108,6 @@ where
 /// Like [`objects()`] but using a single thread only to mostly save on the otherwise required overhead.
 pub fn objects_unthreaded<Find, IterErr, Oid>(
     db: Find,
-    (pack_cache, obj_cache): (&mut impl crate::cache::DecodeEntry, &mut impl crate::cache::Object),
     object_ids: impl Iterator<Item = std::result::Result<Oid, IterErr>>,
     mut progress: impl Progress,
     should_interrupt: &AtomicBool,
@@ -135,11 +128,9 @@ where
         object_ids,
         &mut buf1,
         &mut buf2,
-        pack_cache,
-        obj_cache,
         &mut progress,
         should_interrupt,
-        false,
+        false, /*allow pack lookups*/
     )
 }
 
@@ -168,8 +159,6 @@ mod expand {
         oids: impl IntoIterator<Item = std::result::Result<Oid, IterErr>>,
         buf1: &mut Vec<u8>,
         buf2: &mut Vec<u8>,
-        cache: &mut impl crate::cache::DecodeEntry,
-        obj_cache: &mut impl crate::cache::Object,
         progress: &mut impl Progress,
         should_interrupt: &AtomicBool,
         allow_pack_lookups: bool,
@@ -196,23 +185,28 @@ mod expand {
             }
 
             let id = id.map(|oid| oid.into()).map_err(Error::InputIteration)?;
-            let obj = db.find(id, buf1, cache)?;
+            let (obj, location) = db.find(id, buf1)?;
             stats.input_objects += 1;
             match input_object_expansion {
                 TreeAdditionsComparedToAncestor => {
                     use git_object::Kind::*;
                     let mut obj = obj;
+                    let mut location = location;
                     let mut id = id.to_owned();
 
                     loop {
-                        push_obj_count_unique(&mut out, seen_objs, &id, &obj, progress, stats, false);
+                        push_obj_count_unique(&mut out, seen_objs, &id, location, progress, stats, false);
                         match obj.kind {
                             Tree | Blob => break,
                             Tag => {
                                 id = TagRefIter::from_bytes(obj.data)
                                     .target_id()
                                     .expect("every tag has a target");
-                                obj = db.find(id, buf1, cache)?;
+                                let tmp = db.find(id, buf1)?;
+
+                                obj = tmp.0;
+                                location = tmp.1;
+
                                 stats.expanded_objects += 1;
                                 continue;
                             }
@@ -230,8 +224,10 @@ mod expand {
                                             Err(err) => return Err(Error::CommitDecode(err)),
                                         }
                                     }
-                                    let obj = db.find(tree_id, buf1, cache)?;
-                                    push_obj_count_unique(&mut out, seen_objs, &tree_id, &obj, progress, stats, true);
+                                    let (obj, location) = db.find(tree_id, buf1)?;
+                                    push_obj_count_unique(
+                                        &mut out, seen_objs, &tree_id, location, progress, stats, true,
+                                    );
                                     git_object::TreeRefIter::from_bytes(obj.data)
                                 };
 
@@ -242,11 +238,11 @@ mod expand {
                                         &mut tree_traversal_state,
                                         |oid, buf| {
                                             stats.decoded_objects += 1;
-                                            match db.find(oid, buf, cache).ok() {
-                                                Some(obj) => {
+                                            match db.find(oid, buf).ok() {
+                                                Some((obj, location)) => {
                                                     progress.inc();
                                                     stats.expanded_objects += 1;
-                                                    out.push(output::Count::from_data(oid, &obj));
+                                                    out.push(output::Count::from_data(oid, location));
                                                     obj.try_into_tree_iter()
                                                 }
                                                 None => None,
@@ -259,28 +255,22 @@ mod expand {
                                 } else {
                                     for commit_id in &parent_commit_ids {
                                         let parent_tree_id = {
-                                            let parent_commit_obj = db.find(commit_id, buf2, cache)?;
+                                            let (parent_commit_obj, location) = db.find(commit_id, buf2)?;
 
                                             push_obj_count_unique(
-                                                &mut out,
-                                                seen_objs,
-                                                commit_id,
-                                                &parent_commit_obj,
-                                                progress,
-                                                stats,
-                                                true,
+                                                &mut out, seen_objs, commit_id, location, progress, stats, true,
                                             );
                                             CommitRefIter::from_bytes(parent_commit_obj.data)
                                                 .tree_id()
                                                 .expect("every commit has a tree")
                                         };
                                         let parent_tree = {
-                                            let parent_tree_obj = db.find(parent_tree_id, buf2, cache)?;
+                                            let (parent_tree_obj, location) = db.find(parent_tree_id, buf2)?;
                                             push_obj_count_unique(
                                                 &mut out,
                                                 seen_objs,
                                                 &parent_tree_id,
-                                                &parent_tree_obj,
+                                                location,
                                                 progress,
                                                 stats,
                                                 true,
@@ -295,17 +285,7 @@ mod expand {
                                                 &mut tree_diff_state,
                                                 |oid, buf| {
                                                     stats.decoded_objects += 1;
-                                                    let id = oid.to_owned();
-                                                    match obj_cache.get(&id, buf) {
-                                                        Some(_kind) => git_object::TreeRefIter::from_bytes(buf).into(),
-                                                        None => match db.find_tree_iter(oid, buf, cache).ok() {
-                                                            Some(_) => {
-                                                                obj_cache.put(id, git_object::Kind::Tree, buf);
-                                                                git_object::TreeRefIter::from_bytes(buf).into()
-                                                            }
-                                                            None => None,
-                                                        },
-                                                    }
+                                                    db.find_tree_iter(oid, buf).ok().map(|t| t.0)
                                                 },
                                                 &mut changes_delegate,
                                             )
@@ -324,22 +304,22 @@ mod expand {
                 TreeContents => {
                     use git_object::Kind::*;
                     let mut id = id;
-                    let mut obj = obj;
+                    let mut obj = (obj, location);
                     loop {
-                        push_obj_count_unique(&mut out, seen_objs, &id, &obj, progress, stats, false);
-                        match obj.kind {
+                        push_obj_count_unique(&mut out, seen_objs, &id, obj.1.clone(), progress, stats, false);
+                        match obj.0.kind {
                             Tree => {
                                 traverse_delegate.clear();
                                 git_traverse::tree::breadthfirst(
-                                    git_object::TreeRefIter::from_bytes(obj.data),
+                                    git_object::TreeRefIter::from_bytes(obj.0.data),
                                     &mut tree_traversal_state,
                                     |oid, buf| {
                                         stats.decoded_objects += 1;
-                                        match db.find(oid, buf, cache).ok() {
-                                            Some(obj) => {
+                                        match db.find(oid, buf).ok() {
+                                            Some((obj, location)) => {
                                                 progress.inc();
                                                 stats.expanded_objects += 1;
-                                                out.push(output::Count::from_data(oid, &obj));
+                                                out.push(output::Count::from_data(oid, location));
                                                 obj.try_into_tree_iter()
                                             }
                                             None => None,
@@ -354,28 +334,29 @@ mod expand {
                                 break;
                             }
                             Commit => {
-                                id = CommitRefIter::from_bytes(obj.data)
+                                id = CommitRefIter::from_bytes(obj.0.data)
                                     .tree_id()
                                     .expect("every commit has a tree");
                                 stats.expanded_objects += 1;
-                                obj = db.find(id, buf1, cache)?;
+                                obj = db.find(id, buf1)?;
                                 continue;
                             }
                             Blob => break,
                             Tag => {
-                                id = TagRefIter::from_bytes(obj.data)
+                                id = TagRefIter::from_bytes(obj.0.data)
                                     .target_id()
                                     .expect("every tag has a target");
                                 stats.expanded_objects += 1;
-                                obj = db.find(id, buf1, cache)?;
+                                obj = db.find(id, buf1)?;
                                 continue;
                             }
                         }
                     }
                 }
-                AsIs => push_obj_count_unique(&mut out, seen_objs, &id, &obj, progress, stats, false),
+                AsIs => push_obj_count_unique(&mut out, seen_objs, &id, location, progress, stats, false),
             }
         }
+        outcome.total_objects = out.len();
         Ok((out, outcome))
     }
 
@@ -384,7 +365,7 @@ mod expand {
         out: &mut Vec<output::Count>,
         all_seen: &impl util::InsertImmutable<ObjectId>,
         id: &oid,
-        obj: &crate::data::Object<'_>,
+        location: Option<crate::data::entry::Location>,
         progress: &mut impl Progress,
         statistics: &mut Outcome,
         count_expanded: bool,
@@ -396,7 +377,7 @@ mod expand {
             if count_expanded {
                 statistics.expanded_objects += 1;
             }
-            out.push(output::Count::from_data(id, obj));
+            out.push(output::Count::from_data(id, location));
         }
     }
 
